searchNodes=[{"ref":"Posexional.html","title":"Posexional","type":"module","doc":"Posexional is a library to manage positional files in elixir. Positional files are the most bad and terrifying file format you ever want to work with, believe me, it&#39;s utter crap. Still some web services use this format to expose data services, and if you are here reading this, probably you are in luck like us! A positional file has a format specification like this: field nameoffsetlengthnotes code05request code fill with 0 and align on the right prog05progressive number (fill with spaces) type212AA number24201 name2710person name (fill with -) future use382leave empty end411! The expected results should be something like this: 000B1 1AA01george---- ! 000B2 2AA01john------ ! 000B3 3AA01ringo----- ! 000B4 4AA01paul------ ! cool uh? With Posexional you can produce this file by defining a module that use Posexional defmodule BeatlesFile do use PosexionalFile @separator &quot;\\n&quot; row :beatles do value :code, 5, filler: ?0, alignment: :right progressive_number :code, 5 fixed_value &quot;AA&quot; fixed_value &quot;01&quot; value :name, 10, filler: ?- empty 2 fixed_value &quot;!&quot; end end and then use it in your code BeatlesFile.write([ beatles: [code: &quot;B1&quot;, name: &quot;george&quot;], beatles: [code: &quot;B2&quot;, name: &quot;john&quot;], beatles: [code: &quot;B2&quot;, name: &quot;ringo&quot;], beatles: [code: &quot;B2&quot;, name: &quot;paul&quot;] ]) In the first part we define the structure inside a module. We are not saying what the content or the number of rows there will be, we are just saying that there is a row called :beatles with the structure declared by the fields Then we can call BeatlesFile.write/1, we pass the data that should be written in the fields. Just the relevant data, The empty fields, the fixed values or the progressive number is managed by the library itself. The write/1 function accept a keyword list with the row name as key, and a keyword list of {field name, field value} of data. If some data is bigger than the field size an error is thrown With the same exact module, we can even read a positional file by calling read/1 and passing a binary string of the file content. There is only one thing to notice, when we write a file we can be declarative and say what row we want to write, as well as the data we want in it. On the other hand, while reading a positional file, we don&#39;t know which rows we are reading, so we need to tell in some way to every row how it is recognized, so that the parser is able to do its job Since we only have a row type (the beatles one) we can just say to the module to always match the beatles row, as simple as defmodule BeatlesFile do use PosexionalFile @separator &quot;\\n&quot; row :beatles, :always do # add :always here to always match this row while reading value :code, 5, filler: ?0, alignment: :right progressive_number :code, 5 fixed_value &quot;AA&quot; fixed_value &quot;01&quot; value :name, 10, filler: ?- empty 2 fixed_value &quot;!&quot; end end :always is a special type that always match, you could also pass :never (not so useful!) and, to gain total control over the choice, a function with a single argument(the full row content) to match some data inside of it. Now we are able to parse a positional file &quot;000B1 1AA01george---- ! 000B2 2AA01john------ ! 000B2 3AA01ringo----- ! 000B2 4AA01paul------ !&quot; |&gt; BeatlesFile.read |&gt; IO.puts this is the output [beatles: [code: &quot;B1&quot;, code: 1, fixed_value: &quot;AA&quot;, fixed_value: &quot;01&quot;, name: &quot;george&quot;, fixed_value: &quot;!&quot;], beatles: [code: &quot;B2&quot;, code: 2, fixed_value: &quot;AA&quot;, fixed_value: &quot;01&quot;, name: &quot;john&quot;, fixed_value: &quot;!&quot;], beatles: [code: &quot;B2&quot;, code: 3, fixed_value: &quot;AA&quot;, fixed_value: &quot;01&quot;, name: &quot;ringo&quot;, fixed_value: &quot;!&quot;], beatles: [code: &quot;B2&quot;, code: 4, fixed_value: &quot;AA&quot;, fixed_value: &quot;01&quot;, name: &quot;paul&quot;, fixed_value: &quot;!&quot;]] positional files cool again!!! Well no...they still sucks...but a little less."},{"ref":"Posexional.html#read/2","title":"Posexional.read/2","type":"function","doc":"read a positional stream of data with the given struct, returns a keyword list of the extracted data"},{"ref":"Posexional.html#read_file!/2","title":"Posexional.read_file!/2","type":"function","doc":"same as read/2, but with a path to a file to read the stream from"},{"ref":"Posexional.html#write/2","title":"Posexional.write/2","type":"function","doc":"write a positional file with the given struct and data"},{"ref":"Posexional.html#write_file!/3","title":"Posexional.write_file!/3","type":"function","doc":"same as write/2, but with a path to a new file to write the result to"},{"ref":"Posexional.Field.html","title":"Posexional.Field","type":"module","doc":"generic utility functions for fields"},{"ref":"Posexional.Field.html#contains_only?/2","title":"Posexional.Field.contains_only?/2","type":"function","doc":"true if the value passed contains only the filler value"},{"ref":"Posexional.Field.html#depositionalize/2","title":"Posexional.Field.depositionalize/2","type":"function","doc":""},{"ref":"Posexional.Field.html#nil_if_empty/2","title":"Posexional.Field.nil_if_empty/2","type":"function","doc":"nil if the value is an empty string, or a string containing only the filler"},{"ref":"Posexional.Field.html#positionalize/2","title":"Posexional.Field.positionalize/2","type":"function","doc":"justify a value given alignment, size and filler char, if the given value is longer it gets trimmed Examples iex&gt; Posexional.Field.positionalize(&quot;test&quot;, ...&gt; %{alignment: :left, size: 10, filler: ?\\s}) &quot;test &quot; iex&gt; Posexional.Field.positionalize(&quot;test&quot;, ...&gt; %{alignment: :right, size: 10, filler: ?\\s}) &quot; test&quot; iex&gt; Posexional.Field.positionalize(&quot;test&quot;, ...&gt; %{alignment: :right, size: 5, filler: ?\\s}) &quot; test&quot; iex&gt; Posexional.Field.positionalize(&quot;test&quot;, ...&gt; %{alignment: :right, size: 5, filler: ?-}) &quot;-test&quot; iex&gt; Posexional.Field.positionalize(&quot;testtest&quot;, ...&gt; %{alignment: :right, size: 5, filler: ?-}) &quot;testt&quot;"},{"ref":"Posexional.Field.Empty.html","title":"Posexional.Field.Empty","type":"module","doc":"this module represent a single field in a row of a positional file without a value"},{"ref":"Posexional.Field.Empty.html#new/2","title":"Posexional.Field.Empty.new/2","type":"function","doc":""},{"ref":"Posexional.Field.Empty.html#write/1","title":"Posexional.Field.Empty.write/1","type":"function","doc":"outputs an empty field Examples iex&gt; Posexional.Field.Empty.write(%Posexional.Field.Empty{filler: ?-, size: 10}) &quot;----------&quot; iex&gt; Posexional.Field.Empty.write(%Posexional.Field.Empty{filler: ?\\s, size: 2}) &quot; &quot;"},{"ref":"Posexional.Field.FixedValue.html","title":"Posexional.Field.FixedValue","type":"module","doc":"this module represent a fixed value field it is initialized with its value and it never change"},{"ref":"Posexional.Field.FixedValue.html#new/1","title":"Posexional.Field.FixedValue.new/1","type":"function","doc":""},{"ref":"Posexional.Field.ProgressiveNumber.html","title":"Posexional.Field.ProgressiveNumber","type":"module","doc":"this module represent a single field in a row of a positional file with a progressive number"},{"ref":"Posexional.Field.ProgressiveNumber.html#new/3","title":"Posexional.Field.ProgressiveNumber.new/3","type":"function","doc":""},{"ref":"Posexional.Field.ProgressiveNumber.html#write/2","title":"Posexional.Field.ProgressiveNumber.write/2","type":"function","doc":""},{"ref":"Posexional.Field.Value.html","title":"Posexional.Field.Value","type":"module","doc":"this module represent a single field in a row of a positional file"},{"ref":"Posexional.Field.Value.html#new/3","title":"Posexional.Field.Value.new/3","type":"function","doc":""},{"ref":"Posexional.Field.Value.html#write/2","title":"Posexional.Field.Value.write/2","type":"function","doc":"outputs a field Examples iex&gt; Posexional.Field.Value.write(Posexional.Field.Value.new(:test, 5), &quot;test&quot;) &quot;test &quot; iex&gt; Posexional.Field.Value.write(Posexional.Field.Value.new(:test, 5), &quot;too long&quot;) ** (RuntimeError) The value too long is too long for the test field. The maximum size is 5 while the value is 8 iex&gt; Posexional.Field.Value.write(Posexional.Field.Value.new(:test, 10), &quot;test&quot;) &quot;test &quot; iex&gt; Posexional.Field.Value.write(Posexional.Field.Value.new(:test, 10), &quot;test&quot;) &quot;test &quot; iex&gt; Posexional.Field.Value.write(Posexional.Field.Value.new(:test, 10, filler: ?0), &quot;test&quot;) &quot;test000000&quot; iex&gt; Posexional.Field.Value.write(Posexional.Field.Value.new(:test, 10, filler: ?0, alignment: :right), &quot;test&quot;) &quot;000000test&quot; iex&gt; Posexional.Field.Value.write(Posexional.Field.Value.new(:test, 10), 50) ** (RuntimeError) The value provided for the test field doesn&#39;t seem to be a string"},{"ref":"Posexional.File.html","title":"Posexional.File","type":"module","doc":"a Posexional.File is the main struct to manage a positional file"},{"ref":"Posexional.File.html#find_row/2","title":"Posexional.File.find_row/2","type":"function","doc":""},{"ref":"Posexional.File.html#get_counters/1","title":"Posexional.File.get_counters/1","type":"function","doc":""},{"ref":"Posexional.File.html#manage_counters/1","title":"Posexional.File.manage_counters/1","type":"function","doc":"adds a generator for every progressive_number_field in the file. The fields are grouped by name, so that you can specify many counters for every row"},{"ref":"Posexional.File.html#new/2","title":"Posexional.File.new/2","type":"function","doc":""},{"ref":"Posexional.File.html#read/2","title":"Posexional.File.read/2","type":"function","doc":""},{"ref":"Posexional.File.html#write/2","title":"Posexional.File.write/2","type":"function","doc":"creates a file from values Examples iex&gt; Posexional.File.write( ...&gt; Posexional.File.new([ Posexional.Row.new(:row_test, [ Posexional.Field.Value.new(:test1, 5) ]) ]), ...&gt; [row_test: [test1: &quot;test&quot;], row_test: [test1: &quot;t&quot;]] ...&gt; ) &quot;test \\nt &quot; iex&gt; Posexional.File.write( ...&gt; Posexional.File.new([ Posexional.Row.new(:row_test, [ Posexional.Field.Value.new(:test1, 5) ]) ]), ...&gt; [row_test: [test1: &quot;test&quot;], ne: [test1: &quot;t&quot;]] ...&gt; ) ** (RuntimeError) row ne not found"},{"ref":"Posexional.File.html#write_path!/3","title":"Posexional.File.write_path!/3","type":"function","doc":""},{"ref":"Posexional.Protocol.FieldLength.html","title":"Posexional.Protocol.FieldLength","type":"protocol","doc":""},{"ref":"Posexional.Protocol.FieldLength.html#length/1","title":"Posexional.Protocol.FieldLength.length/1","type":"function","doc":"returns the field length"},{"ref":"Posexional.Protocol.FieldLength.html#t:t/0","title":"Posexional.Protocol.FieldLength.t/0","type":"type","doc":""},{"ref":"Posexional.Protocol.FieldName.html","title":"Posexional.Protocol.FieldName","type":"protocol","doc":""},{"ref":"Posexional.Protocol.FieldName.html#name/1","title":"Posexional.Protocol.FieldName.name/1","type":"function","doc":"returns the field name"},{"ref":"Posexional.Protocol.FieldName.html#t:t/0","title":"Posexional.Protocol.FieldName.t/0","type":"type","doc":""},{"ref":"Posexional.Protocol.FieldRead.html","title":"Posexional.Protocol.FieldRead","type":"protocol","doc":""},{"ref":"Posexional.Protocol.FieldRead.html#read/2","title":"Posexional.Protocol.FieldRead.read/2","type":"function","doc":"returns the value of a field, given it&#39;s positional representation"},{"ref":"Posexional.Protocol.FieldRead.html#t:t/0","title":"Posexional.Protocol.FieldRead.t/0","type":"type","doc":""},{"ref":"Posexional.Protocol.FieldSize.html","title":"Posexional.Protocol.FieldSize","type":"protocol","doc":""},{"ref":"Posexional.Protocol.FieldSize.html#size/1","title":"Posexional.Protocol.FieldSize.size/1","type":"function","doc":"returns the field name"},{"ref":"Posexional.Protocol.FieldSize.html#t:t/0","title":"Posexional.Protocol.FieldSize.t/0","type":"type","doc":""},{"ref":"Posexional.Protocol.FieldWrite.html","title":"Posexional.Protocol.FieldWrite","type":"protocol","doc":""},{"ref":"Posexional.Protocol.FieldWrite.html#write/2","title":"Posexional.Protocol.FieldWrite.write/2","type":"function","doc":"returns the field to be inserted in the positional file"},{"ref":"Posexional.Protocol.FieldWrite.html#t:t/0","title":"Posexional.Protocol.FieldWrite.t/0","type":"type","doc":""},{"ref":"Posexional.Row.html","title":"Posexional.Row","type":"module","doc":"this module represent a row in a positional file"},{"ref":"Posexional.Row.html#add_field/2","title":"Posexional.Row.add_field/2","type":"function","doc":""},{"ref":"Posexional.Row.html#add_fields/2","title":"Posexional.Row.add_fields/2","type":"function","doc":""},{"ref":"Posexional.Row.html#fields_from/2","title":"Posexional.Row.fields_from/2","type":"function","doc":"merge fields from another row"},{"ref":"Posexional.Row.html#find_field/2","title":"Posexional.Row.find_field/2","type":"function","doc":"finds a field in the row by its name Examples iex&gt; Posexional.Row.new(:row_test, []) |&gt; Posexional.Row.find_field(:test) nil iex&gt; Posexional.Row.new(:row_test, [Posexional.Field.Value.new(:test, 5)]) |&gt; Posexional.Row.find_field(:test) Posexional.Field.Value.new(:test, 5) iex&gt; Posexional.Row.new(:row_test, [Posexional.Field.Value.new(:test, 5), Posexional.Field.Value.new(:test2, 5)]) ...&gt; |&gt; Posexional.Row.find_field(:test2) Posexional.Field.Value.new(:test2, 5)"},{"ref":"Posexional.Row.html#length/1","title":"Posexional.Row.length/1","type":"function","doc":"calculate the row total length based on the passed fields Examples iex&gt; Posexional.Row.new(:row_test, []) ...&gt; |&gt; Posexional.Row.length 0 iex&gt; Posexional.Row.new(:row_test, [Posexional.Field.Value.new(:test1, 10), Posexional.Field.Value.new(:test2, 20)]) ...&gt; |&gt; Posexional.Row.length 30"},{"ref":"Posexional.Row.html#manage_counters/2","title":"Posexional.Row.manage_counters/2","type":"function","doc":""},{"ref":"Posexional.Row.html#new/3","title":"Posexional.Row.new/3","type":"function","doc":""},{"ref":"Posexional.Row.html#offset/2","title":"Posexional.Row.offset/2","type":"function","doc":"Given a row and a field name calculate the field offset Examples iex&gt; Posexional.Row.new(:test, [Posexional.Field.Value.new(:test1, 10), Posexional.Field.Value.new(:test2, 20)]) ...&gt; |&gt; Posexional.Row.offset(:test1) 1 iex&gt; Posexional.Row.new(:test, [Posexional.Field.Value.new(:test1, 10), Posexional.Field.Value.new(:test2, 20)]) ...&gt; |&gt; Posexional.Row.offset(:test2) 11 iex&gt; Posexional.Row.new(:test, [Posexional.Field.Value.new(:test1, 10), Posexional.Field.Value.new(:test2, 20)]) ...&gt; |&gt; Posexional.Row.offset(:test_not_existent) ** (ArgumentError) the field test_not_existent doesn&#39;t exists iex&gt; Posexional.Row.new(:test, []) ...&gt; |&gt; Posexional.Row.offset(:test) nil"},{"ref":"Posexional.Row.html#read/2","title":"Posexional.Row.read/2","type":"function","doc":"read a positional file row and convert it back to a keyword list of values"},{"ref":"Posexional.Row.html#write/2","title":"Posexional.Row.write/2","type":"function","doc":"outputs a row Examples iex&gt; Posexional.Row.new(:row_test, []) |&gt; Posexional.Row.write([test: &quot;test&quot;]) {:ok, &quot;&quot;} iex&gt; Posexional.Row.new(:row_test, [Posexional.Field.Value.new(:test1, 5), Posexional.Field.Value.new(:test2, 10)]) ...&gt; |&gt; Posexional.Row.write([test1: &quot;test1&quot;, test2: &quot;test2&quot;]) {:ok, &quot;test1test2 &quot;} iex&gt; Posexional.Row.new(:row_test, [Posexional.Field.Value.new(:test1, 5), Posexional.Field.Value.new(:test2, 10)]) ...&gt; |&gt; Posexional.Row.write([test1: &quot;test1&quot;, non_existent: &quot;test2&quot;]) {:ok, &quot;test1 &quot;} iex&gt; Posexional.Row.new(:row_test, [Posexional.Field.Value.new(:test1, 6)]) ...&gt; |&gt; Posexional.Row.write([test1: &quot;test1&quot;, not_configured: &quot;test2&quot;]) {:ok, &quot;test1 &quot;} iex&gt; Posexional.Row.new(:row_test, [Posexional.Field.Value.new(:test1, 5)]) ...&gt; |&gt; Posexional.Row.write([not_configured: &quot;test2&quot;, another: &quot;test3&quot;]) {:ok, &quot; &quot;} iex&gt; Posexional.Row.new(:row_test, [Posexional.Field.Empty.new(5)]) ...&gt; |&gt; Posexional.Row.write([]) {:ok, &quot; &quot;}"},{"ref":"PosexionalFile.html","title":"PosexionalFile","type":"module","doc":"macros for defining file modules"},{"ref":"PosexionalFile.html#__using__/1","title":"PosexionalFile.__using__/1","type":"macro","doc":"add use Posexional on top of an elixir module to use macros to define fields"},{"ref":"PosexionalFile.html#row/1","title":"PosexionalFile.row/1","type":"macro","doc":""},{"ref":"PosexionalFile.html#row/2","title":"PosexionalFile.row/2","type":"macro","doc":"define a row of a positional file"},{"ref":"PosexionalFile.html#row/3","title":"PosexionalFile.row/3","type":"macro","doc":""},{"ref":"PosexionalRow.html","title":"PosexionalRow","type":"module","doc":"macros for defining row modules"},{"ref":"PosexionalRow.html#__using__/1","title":"PosexionalRow.__using__/1","type":"macro","doc":"add use Posexional on top of an elixir module to use macros to define fields"},{"ref":"PosexionalRow.html#empty/2","title":"PosexionalRow.empty/2","type":"macro","doc":"add an empty field"},{"ref":"PosexionalRow.html#fixed_value/1","title":"PosexionalRow.fixed_value/1","type":"macro","doc":"add a field with a fixed value"},{"ref":"PosexionalRow.html#guesser/1","title":"PosexionalRow.guesser/1","type":"macro","doc":"sets the row matcher"},{"ref":"PosexionalRow.html#import_fields_from/1","title":"PosexionalRow.import_fields_from/1","type":"macro","doc":"add all fields from another row module"},{"ref":"PosexionalRow.html#name/1","title":"PosexionalRow.name/1","type":"macro","doc":"sets the row name, if no name is provided the module name will be used"},{"ref":"PosexionalRow.html#progressive_number/3","title":"PosexionalRow.progressive_number/3","type":"macro","doc":"add a field with a progressive_number value"},{"ref":"PosexionalRow.html#separator/1","title":"PosexionalRow.separator/1","type":"macro","doc":"sets the row separator"},{"ref":"PosexionalRow.html#value/3","title":"PosexionalRow.value/3","type":"macro","doc":"add a value field"}]